'''
Created on 22/07/2009

@author: LUIS.RANGEL
'''

import os, sys
import threading
from functools import partial
from ftplib import Error
from modelo import gestionaSitios
from PyQt4.QtCore import *
from PyQt4.QtGui import *
from utilidades.Coneccion import *
from modelo.sitio import Sitio
from ui.uiSitio import *
from enum import Enum

class hSitio(QThread):
    def __init__(self, sitio, tiempo, evento, parent = None):
        QThread.__init__(self, parent)
        self.sitio = sitio
        self.tiempo = tiempo
        self.evento = evento
        self.continua = True
    
    def run(self):
        while self.continua:
            self.emit(SIGNAL("Procesando"))
            self.sitio.cargaInfo()
            self.evento.clear()
            
            salida = (BD_noFTP, FTP_noBD, BD_FTP) = self.sitio.compara()
            if len(BD_noFTP) > 0 or len(FTP_noBD) > 0:
                self.emit(SIGNAL("Esperando"))
                self.evento.wait()
            else:
                self.emit(SIGNAL("Durmiendo"))
                self.sleep(self.tiempo)
    
    def quit(self):
        self.continua = False
        self.evento.set()
        QThread.quit(self)

class HSitioSinc(QThread):
    def __init__(self, sitio, parent = None):
        QThread.__init__(self, parent)
        self.sitio = sitio
    
    def run(self):
        self.emit(SIGNAL("Procesando"))
        self.sitio.sincroniza()
        self.emit(SIGNAL("Saliendo"))
        pass
        

class UISitios(QWidget):
    cabecera = Enum("Sitio", "Nombre", "URL", "Status", "Acciones")
    
    def __init__(self, parent = None):
        QWidget.__init__(self, parent)
        
        self.trayIcon = QSystemTrayIcon(QIcon(QPixmap(":/review")), self) 
        self.trayIcon.setToolTip("Revisa FTP")
        self.trayIcon.setVisible(True)
        self.connect(self.trayIcon, SIGNAL("activated (QSystemTrayIcon::ActivationReason)"), self.mostrarVentana)
        
        self.vSitios = []
        self.vHilos = []
        self.gSitio = gestionaSitios.GestionaSitios()
        
        self.setWindowTitle("Revisa FTP")
        
        self.tabla = QTableWidget(self)
        self.bEditar = QPushButton("Nuevo")
        self.bRecargar = QPushButton("Recargar")
        self.bSincronizar = QPushButton("Sincronizar")
        self.bEliminar = QPushButton("Eliminar")
        
        self.tabla.setColumnCount(len(self.cabecera))
        self.tabla.setHorizontalHeaderLabels([str(x) for x in self.cabecera])
        
        lBotones = QVBoxLayout()
        lBotones.addWidget(self.bEditar)
        lBotones.addWidget(self.bEliminar)
        lBotones.addWidget(self.bRecargar)
        lBotones.addWidget(self.bSincronizar)
        lBotones.addStretch()
        
        grid = QGridLayout()
        grid.addWidget(self.tabla, 0, 0, 0, 1)
        grid.addWidget(QLabel("xlarsx"), 1, 1, Qt.AlignRight)
        grid.addLayout(lBotones, 0, 1)
        self.setLayout(grid)
        
        self.connect(self.bRecargar, SIGNAL("clicked()"), self.cargaInfo)
        self.connect(self.bEditar, SIGNAL("clicked()"), self.actualizaSitio)
        self.connect(self.bEliminar, SIGNAL("clicked()"), self.eliminaSitio)
        self.connect(self.tabla, SIGNAL("itemSelectionChanged()"), self.cambiaTituloBEditar)
        self.cargaInfo()
        
        self.tabla.resizeColumnsToContents()
    
    def mostrarVentana(self, ActivationReason):
        if ActivationReason == QSystemTrayIcon.Trigger:
            if self.isHidden():
                self.setVisible(True)
                self.resize(self.tamagno)
            else:
                self.tamagno = self.size()
                self.setVisible(False)
    
    def cambiaTituloBEditar(self):
        texto = "Nuevo"
        if len(self.tabla.selectedIndexes()) > 0:
            texto = "Editar"
        
        self.bEditar.setText(texto)
    
    def actualizaSitio(self):
        recargar = False
        indices = self.tabla.selectedIndexes()
        renglonesTrabajados = []
        try:
            if len(indices) > 0:
                for indice in indices:
                    renglon = indice.row()
                    if renglon not in renglonesTrabajados:
                        renglonesTrabajados.append(renglon)
                        
                        if len(self.vSitios) > renglon and renglon >= 0:
                            sitio = self.vSitios[renglon]
                            uisitio = UISitio(sitio, self)
                            if uisitio.exec_():
                                sitio = uisitio.obtenInfo()
                                Sitio.actualizaSitio(sitio)
                                recargar = True
            else:
                sitio = AtributosSitio()
                uisitio = UISitio(sitio, self)
                if uisitio.exec_():
                    sitio = uisitio.obtenInfo()
                    Sitio.actualizaSitio(sitio)
                    recargar = True
        except ValueError, e:
            QMessageBox.critical(self, "Error", "Datos Incorrectos, cambiados insatisfactorio", QMessageBox.Ok, QMessageBox.Ok)
        except Error, e:
            QMessageBox.critical(self, "Error", "Datos Incorrectos, cambiados insatisfactorio", QMessageBox.Ok, QMessageBox.Ok)
        
        if recargar:
            QMessageBox.information(self, "Info", "Sitio agregado satisfactoriamente", QMessageBox.Ok, QMessageBox.Ok)
            self.cargaInfo()
    
    def eliminaSitio(self):
        if QMessageBox.warning(self, "Eliminar Sitio", "Esta seguro de eliminar el sitio?", QMessageBox.Ok | QMessageBox.Cancel, QMessageBox.Cancel) == QMessageBox.Ok:
            indices = self.tabla.selectedIndexes()
            renglonesTrabajados = []
            if len(indices) > 0:
                for indice in indices:
                    renglon = indice.row()
                    if renglon not in renglonesTrabajados:
                        renglonesTrabajados.append(renglon)
                        if len(self.vSitios) > renglon and renglon >= 0:
                            sitio = self.vSitios[renglon]
                            if self.gSitio.eliminaSitio(sitio):
                                QMessageBox.information(self, "Info", "Sitio %s eliminado satisfactoriamente" % sitio.nombre, QMessageBox.Ok, QMessageBox.Ok)
                                self.cargaInfo()
    
    def cargaInfo(self):
        if len(self.vHilos) > 0:
            for hilo in self.vHilos:
                hilo.quit()
        self.vHilos = []
        self.vSitios = self.gSitio.listaSitios()
        self.tabla.setRowCount(len(self.vSitios))
        
        cSitio = 0
        for sitio in self.vSitios:
            iSitio = QTableWidgetItem(str(sitio.idSitio))
            iNombre = QTableWidgetItem(sitio.nombre)
            iURL = QTableWidgetItem(sitio.URL)
            status = QLabel("Procesando")
            
            panelAcciones = QWidget(self)
            layoutAcciones = QHBoxLayout()
            bSincronizar = QPushButton("Sincronizar")
            bReiniciar = QPushButton("Reiniciar")
            bDetalle = QPushButton("Detalle")
            layoutAcciones.addWidget(bSincronizar)
            layoutAcciones.addWidget(bDetalle)
            layoutAcciones.addWidget(bReiniciar)
            panelAcciones.setLayout(layoutAcciones)
            
            iSitio.setFlags(Qt.ItemIsSelectable | Qt.ItemIsEnabled)
            iNombre.setFlags(Qt.ItemIsSelectable | Qt.ItemIsEnabled)
            iURL.setFlags(Qt.ItemIsSelectable | Qt.ItemIsEnabled)
            
            self.tabla.setItem(cSitio, self.cabecera.Sitio.index, iSitio)
            self.tabla.setItem(cSitio, self.cabecera.Nombre.index, iNombre)
            self.tabla.setItem(cSitio, self.cabecera.URL.index, iURL)
            self.tabla.setCellWidget(cSitio, self.cabecera.Status.index, status)
            self.tabla.setCellWidget(cSitio, self.cabecera.Acciones.index, panelAcciones)
            
            evento = threading.Event()
            s = Sitio(sitio)
            hsitio = hSitio(s, 600, evento=evento, parent=self)
            hss = HSitioSinc(s, self)
            self.vHilos.append(hsitio)
            self.vHilos.append(hss)
            
            #Procesando
            self.connect(hsitio, SIGNAL("Procesando"), partial(status.setText, "Procesando"))
            self.connect(hsitio, SIGNAL("Procesando"), partial(iNombre.setBackgroundColor, QColor("white")))
            self.connect(hsitio, SIGNAL("Procesando"), partial(bSincronizar.setEnabled, False))
            self.connect(hsitio, SIGNAL("Procesando"), partial(bReiniciar.setEnabled, False))
            
            #Esperando
            self.connect(hsitio, SIGNAL("Esperando"), partial(status.setText, "Modificado"))
            self.connect(hsitio, SIGNAL("Esperando"), partial(iNombre.setBackgroundColor, QColor("yellow")))
            self.connect(hsitio, SIGNAL("Esperando"), partial(bSincronizar.setEnabled, True))
            self.connect(hsitio, SIGNAL("Esperando"), partial(bReiniciar.setEnabled, True))
            self.connect(hsitio, SIGNAL("Esperando"), partial(self.trayIcon.showMessage, "pRevisaFTP", str("Sitio " + sitio.nombre + " esperando")))
            
            #Durmiendo
            self.connect(hsitio, SIGNAL("Durmiendo"), partial(status.setText, "Identico"))
            
            #Botones
            self.connect(self.bSincronizar, SIGNAL("clicked()"), hss.start)
            self.connect(self.bSincronizar, SIGNAL("clicked()"), evento.set)
            self.connect(bReiniciar, SIGNAL("clicked()"), evento.set)
            self.connect(bSincronizar, SIGNAL("clicked()"), hss.start)
            self.connect(bDetalle, SIGNAL("clicked()"), partial(s.comparaArchivosUI, hss))
            self.connect(bDetalle, SIGNAL("clicked()"), evento.set)
            self.connect(hsitio, SIGNAL("terminated()"), self.imprime)
            
            self.connect(hss, SIGNAL("started()"), partial(bSincronizar.setEnabled, False))
            self.connect(hss, SIGNAL("started()"), partial(bSincronizar.setEnabled, False))
            self.connect(hss, SIGNAL("finished()"), evento.set)
            
            hsitio.start()
            self.tabla.setRowHeight(cSitio, 50)
            
            cSitio = cSitio + 1
        
        self.tabla.resizeColumnsToContents()
    
    def imprime(self):
        print "Termine"